typedef struct {
	int16_t x,y;
} Location;
	
struct lock objectsLock;

//gearboxes for capturing the regions.
Location gearbox[6] = { {.x=1791, .y=443}, {.x=512, .y=1773}, {.x=-1280, .y=1330}, {.x=-1791, .y=-443}, {.x=-512, .y=-1773}, {.x=1280, .y=-1330}};

//mines/dispensers to get the balls.
Location mine[6] = { {.x=1791, .y=-443}, {.x=1280, .y=1330}, {.x=-512, .y=1773}, {.x=-1791, .y=443}, {.x=-1280, .y=-1330}, {.x=512, .y=-1773} };

// dispensing sites = the mid points on the inner hexagon
Location dispenser[6] = { {.x=443, .y=0}, {.x=222, .y=384}, {.x=-222, .y=384}, {.x=-443, .y=0}, {.x=-222, .y=-384}, {.x=222, .y=-384} };

// midpoints ( on the mid hexagon
//Location midpoint[6] = { {.x=1245, .y=0}, {.x=623, .y=1078}, {.x=-623, .y=1078}, {.x=-1245, .y=0}, {.x=-623, .y=-1078}, {.x=401, .y=-1078}} ;

// midpoints between the edges of the outer hexagon and the origin
Location midpoint[6] = { {.x=1024, .y=0}, {.x=512, .y=887}, {.x=-512, .y=887}, {.x=-1024, .y=0}, {.x=-512, .y=-887}, {.x=512, .y=-887} };

Location dispenserrr[6], mids[6];

void VPSCorrect (Location *loc, float theta) { 
	loc->x = loc->x - (loc->x*ROBOT_HEIGHT/VPS_HEIGHT);
	loc->y = loc->y - (loc->y*ROBOT_HEIGHT/VPS_HEIGHT);
}


Location getObjects(int ind, float *theta, uint32_t *time)
{
	Location loc;
	float angle;
	
	acquire(&objectsLock);
	copy_objects();
	
	loc.x = game.coords[ind].x;
	loc.y = game.coords[ind].y;
	angle = convertVPSAngleToDegees(game.coords[ind].theta);
	if(time!=NULL) *time = position_microtime;

	release(&objectsLock);
	
	if(theta!=NULL) *theta = angle;
	//VPSCorrect(&loc, angle);
	
	//loc.x -= 37*cos(angle*PI/180);
	//loc.y -= 37*sin(angle*PI/180);
	
	return loc;
}

Location getVPSCurrentLocation(float *theta, uint32_t *time)
{
	return getObjects(0, theta, time);
}
Location getOpponentLocation()
{
	return getObjects(1, NULL, NULL);
}
float getVPSHeading()
{
	float theta;
	getVPSCurrentLocation(&theta, NULL);
	return theta;
}
uint8_t isCaptured(uint8_t region){
	copy_objects();
	return (game.territories[region].owner==robot_id);
}
uint8_t remainingBalls(uint8_t region){
	copy_objects();
	return (game.territories[region].remaining);
}
uint8_t rateLimit(uint8_t region){
	copy_objects();
	return (game.territories[region].rate_limit);
}
uint8_t isAligned(){
	return (digital_read(1)==1 || digital_read(4)==1);
}
Location midPoint(Location *from, Location *to) {
	Location midPoint;
	midPoint.x=(int32_t)((to->x)+(from->x))/2;
	midPoint.y=(int32_t)((to->y)+(from->y))/2;
	return midPoint;
}

float getDistance(Location *from, Location *to) {
	return sqrt((int32_t)((to->x)-(from->x))*((to->x)-(from->x)) + (int32_t)((to->y)-(from->y))*((to->y)-(from->y)));
}
